<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace FiloBlu\Referrals\Controller\Customer\Account;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\App\Action\Context;
use Magento\Customer\Model\Account\Redirect as AccountRedirect;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Customer\Api\AccountManagementInterface;
use Magento\Customer\Helper\Address;
use Magento\Framework\UrlFactory;
use Magento\Customer\Model\Metadata\FormFactory;
use Magento\Newsletter\Model\SubscriberFactory;
use Magento\Customer\Api\Data\RegionInterfaceFactory;
use Magento\Customer\Api\Data\AddressInterfaceFactory;
use Magento\Customer\Api\Data\CustomerInterfaceFactory;
use Magento\Customer\Model\Url as CustomerUrl;
use Magento\Customer\Model\Registration;
use Magento\Framework\Escaper;
use Magento\Customer\Model\CustomerExtractor;
use Magento\Invitation\Model\InvitationProvider;
use Magento\Invitation\Model\Invitation;
use Magento\Framework\Exception\LocalizedException as FrameworkException;
use Magento\Store\Model\ScopeInterface;
use FiloBlu\Referrals\Helper\Core;
use Magento\Invitation\Model\InvitationFactory;
use Magento\SalesRule\Model\ResourceModel\Rule\CollectionFactory;

/**
 * Post create customer action
 *
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class CreatePost extends \Magento\Invitation\Controller\Customer\Account\CreatePost
{
    const RULE_ID_SENDER_OF_THE_INVITE = 'filoblu_referrals/general/ruleidsender';
    const RULE_ID_RECEIVER_OF_THE_INVITE = 'filoblu_referrals/general/ruleidreceiver';

    protected $coreHelper;
    protected $invitationFactory;
    protected $ruleCollectionFactory;

    /**
     * @param Context $context
     * @param Session $customerSession
     * @param ScopeConfigInterface $scopeConfig
     * @param StoreManagerInterface $storeManager
     * @param AccountManagementInterface $accountManagement
     * @param Address $addressHelper
     * @param UrlFactory $urlFactory
     * @param FormFactory $formFactory
     * @param SubscriberFactory $subscriberFactory
     * @param RegionInterfaceFactory $regionDataFactory
     * @param AddressInterfaceFactory $addressDataFactory
     * @param CustomerInterfaceFactory $customerDataFactory
     * @param CustomerUrl $customerUrl
     * @param Registration $registration
     * @param Escaper $escaper
     * @param CustomerExtractor $customerExtractor
     * @param DataObjectHelper $dataObjectHelper
     * @param AccountRedirect $accountRedirect
     * @param InvitationProvider $invitationProvider
     *
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        Context $context,
        Session $customerSession,
        ScopeConfigInterface $scopeConfig,
        StoreManagerInterface $storeManager,
        AccountManagementInterface $accountManagement,
        Address $addressHelper,
        UrlFactory $urlFactory,
        FormFactory $formFactory,
        SubscriberFactory $subscriberFactory,
        RegionInterfaceFactory $regionDataFactory,
        AddressInterfaceFactory $addressDataFactory,
        CustomerInterfaceFactory $customerDataFactory,
        CustomerUrl $customerUrl,
        Registration $registration,
        Escaper $escaper,
        CustomerExtractor $customerExtractor,
        DataObjectHelper $dataObjectHelper,
        AccountRedirect $accountRedirect,
        InvitationProvider $invitationProvider,
        Core $coreHelper,
        InvitationFactory $invitationFactory,
        CollectionFactory $ruleCollectionFactory
    ) {
        $this->coreHelper = $coreHelper;
        $this->invitationFactory = $invitationFactory;
        $this->ruleCollectionFactory = $ruleCollectionFactory;
        parent::__construct(
            $context,
            $customerSession,
            $scopeConfig,
            $storeManager,
            $accountManagement,
            $addressHelper,
            $urlFactory,
            $formFactory,
            $subscriberFactory,
            $regionDataFactory,
            $addressDataFactory,
            $customerDataFactory,
            $customerUrl,
            $registration,
            $escaper,
            $customerExtractor,
            $dataObjectHelper,
            $accountRedirect,
            $invitationProvider
        );
    }

    /**
     * Create customer account action
     *
     * @return \Magento\Framework\Controller\Result\Redirect
     */
    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();

        try {
            if($this->coreHelper->getShareLinkType() != "copypaste") {
                $this->invitationProvider->get($this->getRequest());
            }
            
            \Magento\Customer\Controller\Account\CreatePost::execute();

            if($this->coreHelper->getShareLinkType() == "copypaste") {
                $invitationCode = $this->getRequest()->getParam('invitation');
                $referralCustomer = $this->coreHelper->getCustomerByInvitationCode($invitationCode);
                $invitation = $this->invitationFactory->create();
                $invitation->setData(
                    ['email' => $this->getRequest()->getParam('email'), 'customer' => $referralCustomer, 'message' => '']
                )->save();

                $invitationCoupon = $this->getInvitationCoupons();
                if(count($invitationCoupon) > 0) {
                    foreach($invitationCoupon as $couponField => $couponCode) {
                        $invitation->setData($couponField, $couponCode);
                    }
                    $invitation->save();
                }

                $invitation->accept($this->storeManager->getWebsite()->getId(), $referralCustomer->getId());
            }

            $resultRedirect->setPath('customer/account/');
            return $resultRedirect;
        } catch (\Magento\Framework\Exception\AlreadyExistsException $e) {
            $this->messageManager->addError($e->getMessage());
            $this->session->setCustomerFormData($this->getRequest()->getPostValue());
        } catch (\Magento\Framework\Exception\InputException $e) {
            $this->messageManager->addError($e->getMessage());
            $this->session->setCustomerFormData($this->getRequest()->getPostValue());
        } catch (FrameworkException $e) {
            if ($this->registration->isAllowed()) {
                $this->messageManager->addError(__('Your invitation is not valid. Please create an account.'));
                $resultRedirect->setPath('customer/account/create');
                return $resultRedirect;
            } else {
                $this->messageManager->addError(
                    __(
                        'Your invitation is not valid. Please contact us at %1.',
                        $this->scopeConfig->getValue(
                            'trans_email/ident_support/email',
                            ScopeInterface::SCOPE_STORE
                        )
                    )
                );
                $resultRedirect->setPath('customer/account/login');
                return $resultRedirect;
            }
        } catch (\Exception $e) {
            $this->session->setCustomerFormData($this->getRequest()->getPostValue());
            $this->messageManager->addException($e, __('We can\'t save this customer.'));
        }

        $resultRedirect->setPath('magento_invitation/customer_account/create', ['_current' => true, '_secure' => true]);
        return $resultRedirect;
    }

    private function getInvitationCoupons() {
        $invitationCoupons = array();
        $sender_db_field = $this->coreHelper->getCouponDbFieldNameForWhoSentTheInvite();
        $receiver_db_field = $this->coreHelper->getCouponDbFieldNameForWhoGotInvited();

        $this->coreHelper->writeLog("Preparing the coupons!");

        // Generating and saving the coupon code (magento_invitation table) for the user that send the invite
        $rule_id = $this->coreHelper->getSysConfigValue(self::RULE_ID_SENDER_OF_THE_INVITE,'store',$this->storeManager->getStore()->getId());
        $rule_loaded = $this->loadSalesRuleById($rule_id);
        if($rule_loaded && $rule_loaded->getRuleId()){
            $coupon_code_generated = $this->coreHelper->generateCoupon($rule_loaded->getRuleId());
            if($coupon_code_generated){
                $invitationCoupons[$sender_db_field] = $coupon_code_generated;
                $this->coreHelper->writeLog("Success: generated the coupon {$coupon_code_generated} for ".$sender_db_field);
            } else {
                $this->coreHelper->writeLog("Error: Coupon for ".$sender_db_field." NOT generated!");
            }
        } else {
            $this->coreHelper->writeLog("Error: Salesrule ID ".$rule_id." NOT found!");
        }

        // Generating and saving the coupon code (magento_invitation table) for the user that is invited
        $rule_id = $this->coreHelper->getSysConfigValue(self::RULE_ID_RECEIVER_OF_THE_INVITE,'store',$this->storeManager->getStore()->getId());
        $rule_loaded = $this->loadSalesRuleById($rule_id);
        if($rule_loaded && $rule_loaded->getRuleId()){
            $coupon_code_generated = $this->coreHelper->generateCoupon($rule_loaded->getRuleId());
            if($coupon_code_generated){
                $invitationCoupons[$receiver_db_field] = $coupon_code_generated;
                $this->coreHelper->writeLog("Success: generated the coupon {$coupon_code_generated} for ".$receiver_db_field);
            } else {
                $this->coreHelper->writeLog("Error: Coupon for ".$receiver_db_field." NOT generated!");
            }
        } else {
            $this->coreHelper->writeLog("Error: Salesrule ID ".$rule_id." NOT found!");
        }

        return $invitationCoupons;
    }

    protected function loadSalesRuleById($id = null){
        if(!$id || !is_numeric($id)){
            return false;
        }

        $collection = $this->ruleCollectionFactory->create();
        $collection->addFieldToSelect("*");
        $collection->addFieldToFilter('rule_id', $id);

        if($collection->getSize()){
            $rule_found = $collection->getFirstItem();
            return $rule_found;
        } else {
            return false;
        }
    }
}
